/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     | Website:  https://openfoam.org
    \\  /    A nd           | Copyright (C) 2011-2018 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "structureModel.H"


// * * * * * * * * * * * * * * * * Selector  * * * * * * * * * * * * * * * * //

Foam::autoPtr<Foam::structureModel> Foam::structureModel::New
(
    const dictionary& dict,
    const fvMesh& mesh
)
{
    word type(dict.lookup("type"));

    Info<< endl << "Constructing structure of type: " << type << endl;

    structureModelsConstructorTable::iterator cstrIter =
        structureModelsConstructorTablePtr_->find(type);

    if (cstrIter == structureModelsConstructorTablePtr_->end())
    {
        FatalErrorInFunction
            << "Unknown structure of type "
            << type << endl << endl
            << "Valid structure types are : " << endl
            << structureModelsConstructorTablePtr_->sortedToc()
            << exit(FatalError);
    }

    return cstrIter()(dict, mesh);
}


// ************************************************************************* //
