/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     | Website:  https://openfoam.org
    \\  /    A nd           | Copyright (C) 2011-2018 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "Novendstern.H"
#include "addToRunTimeSelectionTable.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace dragModels
{
    defineTypeNameAndDebug(Novendstern, 0);
    addToRunTimeSelectionTable(dragModel, Novendstern, FSDragModels);
}
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::dragModels::Novendstern::Novendstern
(
    const objectRegistry& objReg,
    const dictionary& dict,
    const FSPair& FSPair
)
:
    dragModel
    (
        objReg,
        dict,
        FSPair
    )
{
    scalar Dp(this->get<scalar>("pinDiameter"));
    scalar Dw(this->get<scalar>("wireDiameter"));
    scalar Lw(this->get<scalar>("wireLeadLen"));
    
    scalar Pt(Dp+1.0444*Dw); // Pitch (pinDia+1.0444*wireDia)
    scalar PbyD(Pt/Dp); // P/D ratio

    A_ = 1.034/pow(PbyD,0.124); 
    B_ = 29.7*(pow(PbyD,6.94)/pow((Lw/Dp),2.239));

}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

Foam::scalar Foam::dragModels::Novendstern::fd(const scalar& Re) const
{
    return (pow((A_+B_*pow(Re, 0.086)),0.885)*0.316/pow(Re, 0.25));
}

void Foam::dragModels::Novendstern::correctKd(volTensorField& Kd) const
{    
    #include "calcKdFromFd.H"
}


// ************************************************************************* //
