/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     | Website:  https://openfoam.org
    \\  /    A nd           | Copyright (C) 2011-2018 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::dragModels::Novendstern

Description
    Novendstern correlation for pressure drop

        @article{Novendstern1972,
	author = {Novendstern, E. H.},
	doi = {10.1016/0029-5493(72)90059-3},
	file = {:C$\backslash$:/Users/ChirayuBatra/workingDirectory/Chirayu/PhD/subChannel/Novendstern.pdf:pdf},
	issn = {00295493},
	journal = {Nuclear Engineering and Design},
	number = {1},
	pages = {28--42},
	title = {{Turbulent flow pressure drop model for fuel rod assemblies utilizing a helical wire-wrap spacer system}},
	volume = {22},
	year = {1972}
	}


Dictionary entries (mandatory)
    pinDiameter         0.006000;
    pinPitch            0.007600;
    wireDiameter        0.0016;
    wireLeadLen         0.1;        //- I.e. pitch of the wrapper wire

SourceFiles
    Novendstern.C

\*---------------------------------------------------------------------------*/

#ifndef Novendstern_H
#define Novendstern_H

#include "dragModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

namespace dragModels
{

/*---------------------------------------------------------------------------*\
                           Class Novendstern Declaration
\*---------------------------------------------------------------------------*/

class Novendstern
:
    public dragModel
{
private:

    scalar A_;
    scalar B_;

public:

    TypeName("Novendstern");

    //- Constructors

        //- Construct for a fluid-structure pair
        Novendstern
        (
            const objectRegistry& objReg,
            const dictionary& dict,
            const FSPair& FSPair
        );

        //- Destructor
        virtual ~Novendstern(){}


    //- Member Functions

        //- Drag coefficient
        virtual void correctKd(volTensorField& Kd) const;

        scalar fd(const scalar& Re) const;
};

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace dragModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
