/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     | Website:  https://openfoam.org
    \\  /    A nd           | Copyright (C) 2011-2018 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "ChenKalish.H"
#include "fluid.H"
#include "addToRunTimeSelectionTable.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace twoPhaseDragMultiplierModels
{
    defineTypeNameAndDebug(ChenKalish, 0);
    addToRunTimeSelectionTable
    (
        twoPhaseDragMultiplierModel, 
        ChenKalish, 
        twoPhaseDragMultiplierModels
    );
}
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::twoPhaseDragMultiplierModels::ChenKalish::ChenKalish
(
    const objectRegistry& objReg,
    const dictionary& dict,
    const fvMesh& mesh
)
:
    twoPhaseDragMultiplierModel
    (
        objReg,
        dict,
        mesh
    )
{}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::twoPhaseDragMultiplierModels::ChenKalish::~ChenKalish()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void
Foam::twoPhaseDragMultiplierModels::ChenKalish::correct()
{
    volScalarField phi2
    (
        IOobject
        (
            "",
            mesh_.time().timeName(),
            this->db()
        ),
        mesh_,
        dimensionedScalar("", dimless, 1)
    );

    const volScalarField& X(mFluid_.XLM());
    forAll(X, i)
    {
        //- I am limiting this for 7e-2 < X < 30 like Kottowski-Savatteri
        //  out of consistency
        scalar logX(log(min(max(X[i], 0.07), 30)));
        phi2[i] = 
            exp
            (
                2.0*
                (
                    0.0867*sqr(logX)
                -   0.518*logX
                +   1.59
                )
            );
    }

    //- This under-relaxes phi2_ and limits its extrema
    this->setPhi2
    (
        phi2
    );
}


// ************************************************************************* //
